@@ -47,9 +47,20 @@ class Topics(environment.DictEnvironment):
         penalty.
     boredom_penalty : float
         The penalty on the rating when a user is bored
+    satiation_factor: float
+        The extent to which satiation affects user ratings.
+    satiation_decay: float
+        A number between 0 and 1 that indicates how quickly satiation decays.
+    satiation_noise: float
+        The standard deviation of the noise influencing satiation at each timestep.
     user_dist_choice : str
         The choice of user distribution for selecting online users. By default, the subset of
         online users is chosen from a uniform distribution. Currently supports normal and lognormal.
+    initial_sampling: str or array
+        How the initial ratings should be sampled. Can be 'uniform', 'powerlaw', or an
+        array of tuples where arr[i][0] and arr[i][1] are the user-id and item-id respectively
+        of the i-th initial rating. If initial_sampling is a string, then users are sampled
+        according to user_dist_choice and items are sampled according to initial_sampling.
     shift_steps : int
         The number of timesteps to wait between each user preference shift.
     shift_frequency : float
@@ -79,14 +90,22 @@ class Topics(environment.DictEnvironment):
                  memory_length=0,
                  boredom_threshold=0,
                  boredom_penalty=0.0,
+                 satiation_factor=0.0,
+                 satiation_decay=0.0,
+                 satiation_noise=0.0,
                  user_dist_choice='uniform',
+                 initial_sampling='uniform',
                  shift_steps=1,
                  shift_frequency=0.0,
                  shift_weight=0.0,
-                 user_bias_type='normal',
-                 item_bias_type='normal'):
+                 user_bias_type='none',
+                 item_bias_type='none'):
         """Create a Topics environment."""
-        super().__init__(rating_frequency, num_init_ratings, memory_length, user_dist_choice)
+        super().__init__(rating_frequency=rating_frequency,
+                         num_init_ratings=num_init_ratings,
+                         memory_length=memory_length,
+                         user_dist_choice=user_dist_choice,
+                         initial_sampling=initial_sampling)
         self._num_topics = num_topics
         self._num_users = num_users
         self._num_items = num_items
@@ -96,6 +115,10 @@ class Topics(environment.DictEnvironment):
         self._item_topics = None
         self._boredom_threshold = boredom_threshold
         self._boredom_penalty = boredom_penalty
+        self._satiation_factor = satiation_factor
+        self._satiation_decay = satiation_decay
+        self._satiation_noise = satiation_noise
+        self._satiations = None
         self._shift_steps = shift_steps
         self._shift_frequency = shift_frequency
         self._shift_weight = shift_weight
@@ -114,6 +137,7 @@ class Topics(environment.DictEnvironment):
         for item_id in range(self._num_items):
             topic = self._item_topics[item_id]
             ratings[:, item_id] = (self._user_preferences[:, topic] +
+                                   self._satiation_factor * self._satiations[:, topic] +
                                    np.full((self._num_users), self._item_biases[item_id]) +
                                    self._user_biases + np.full((self._num_users), self._offset))
 
@@ -129,9 +153,12 @@ class Topics(environment.DictEnvironment):
 
     def _get_rating(self, user_id, item_id):  # noqa: D102
         topic = self._item_topics[item_id]
-        rating = (self._user_preferences[user_id, topic] + self._user_biases[user_id] +
-                  self._item_biases[item_id] + self._offset)
+        rating = (self._user_preferences[user_id, topic] +
+                  self._satiation_factor * self._satiations[user_id, topic] +
+                  self._user_biases[user_id] + self._item_biases[item_id] + self._offset)
         recent_topics = [self._item_topics[item] for item in self._user_histories[user_id]]
+        if len(recent_topics) > 0:
+            recent_topics = list(np.concatenate(recent_topics))
         if recent_topics.count(topic) > self._boredom_threshold:
             rating -= self._boredom_penalty
         rating = np.clip(rating + self._dynamics_random.randn() * self._noise, 1, 5)
@@ -141,32 +168,45 @@ class Topics(environment.DictEnvironment):
         # TODO: Add support for slates of size greater than 1.
         item_id = [item_ids[0]]
         rating = self._get_rating(user_id, item_id)
-        # Updating underlying preference
         topic = self._item_topics[item_id]
+
+        # Update satiation.
+        recommended = np.zeros(self._num_topics)
+        recommended[topic] = 1
+        self._satiations = (self._satiation_decay * (self._satiations + recommended) +
+                            np.random.randn(self._num_topics) * self._satiation_noise)
+
+        # Update underlying preference.
         preference = self._user_preferences[user_id, topic]
         if preference <= 5:
             self._user_preferences[user_id, topic] += self._topic_change
             not_topic = np.arange(self._num_topics) != topic
             self._user_preferences[user_id, not_topic] -= (
                 self._topic_change / (self._num_topics - 1))
+
         return rating
 
     def _reset_state(self):  # noqa: D102
         if self._user_bias_type == 'normal':
             self._user_biases = self._init_random.normal(loc=0., scale=0.5, size=self._num_users)
         elif self._user_bias_type == 'power':
-            self._user_biases = 1-self._init_random.power(5, size=self._num_users)
+            self._user_biases = 1 - self._init_random.power(5, size=self._num_users)
+        elif self._user_bias_type == 'none':
+            self._user_biases = np.zeros(self._num_users)
         else:
             print('User bias distribution is not supported')
 
         if self._item_bias_type == 'normal':
             self._item_biases = self._init_random.normal(loc=0., scale=0.5, size=self._num_items)
         elif self._item_bias_type == 'power':
-            self._item_biases = 1-self._init_random.power(5, size=self._num_users)
+            self._item_biases = 1 - self._init_random.power(5, size=self._num_users)
+        elif self._item_bias_type == 'none':
+            self._item_biases = np.zeros(self._num_items)
         else:
             print('Item bias distribution is not supported')
 
         self._offset = 0
+        self._satiations = np.zeros((self._num_users, self._num_topics))
         self._user_preferences = self._init_random.uniform(low=0.5, high=5.5,
                                                            size=(self._num_users, self._num_topics))
         self._item_topics = self._init_random.choice(self._num_topics, size=self._num_items)
@@ -178,17 +218,17 @@ class Topics(environment.DictEnvironment):
     def _update_state(self):  # noqa: D102
         if (self._timestep + 1) % self._shift_steps == 0:
             # Apply preference and bias shift to a fraction of users.
-
             shifted_users = self._dynamics_random.choice(
                 self._num_users, int(self._num_users * self._shift_frequency))
-
             new_preferences = self._init_random.uniform(low=0.5, high=5.5,
                                                         size=(len(shifted_users), self._num_topics))
             if self._user_bias_type == 'normal':
                 new_user_biases = self._init_random.normal(loc=0, scale=0.5,
                                                            size=len(shifted_users))
             elif self._user_bias_type == 'power':
-                new_user_biases = 1-self._init_random.power(5, size=len(shifted_users))
+                new_user_biases = 1 - self._init_random.power(5, size=len(shifted_users))
+            elif self._user_bias_type == 'none':
+                new_user_biases = np.zeros(self._num_users)
             else:
                 print('User bias distribution is not supported')
 
@@ -198,6 +238,6 @@ class Topics(environment.DictEnvironment):
 
             self._user_biases[shifted_users] = (
                 self._shift_weight * self._user_biases[shifted_users] +
-                (1 - self._shift_weight) * new_user_biases)
+                (1 - self._shift_weight) * new_user_biases[shifted_users])
 
         return collections.OrderedDict(), collections.OrderedDict()
